/*************************************************************************
 * Statistics WS at DESY                                                 
 * Generate toy experiments for 
 * Measurement of the vertical position of a horizontal flying particle
 * as the average position measured in Ndet Detectors (with the same resolution)
 * For simplicity the true track position is always zero.
 * The results are obtained by fitting a p0 for each track
 * Problem: the detectors are noisy -  
 * for each detector in a fraction frac_noise instead of a signal hit a flat
 * distributed noise hit is observed
 *          
 * 
 *    ^                 Det_1  Det_2  Det_3 ...   Det_Ndet
 *  y |                  |      |      o           |
 *    |                  |      |      |           |
 *    | ----> particle   o      |      |           o        
 *    |                  |      o      |           |
 *    |                  |      |      |           |
 *    |
 *     ----------------------> 
 *                         x 
 *
 * Steering parameters:
 * - Ntra: Number of repeated toy experiments
 * - Ndet: Number of detector layers
 * - yerr: Position resolution of the detectors
 * - frac_noise: average fraction of noise hits which are distributed
 *               flatly in a window of size +-10*yerr 
 * Output = Histograms: 
 * - p0d: Deviation of measured average position from true (= Residual)
 * - chi2d: Chi2 distribution from the p0-fits
 * - pchi2d: Chi2-probability distribution from the p0-fits
 *
 * Author: Olaf Behnke                                                   
 *************************************************************************/
#include <stdlib>
#include "TGraphErrors.h"
#include  <TRandom1.h>
#include "TF1.h"
#include "TLine.h"
#include "TMath.h"

const Int_t Ntra=1000;
const Int_t Ndet=10;
const Double_t yerr=0.01; const Double_t yerr2=yerr*yerr;
const Double_t frac_noise=0.1;
//***** const Double_t frac_noise=0.1;
const Double_t xmin = 0.5;
const Double_t xmax = xmin+Ndet;


void p0toyf_sol() {
// Random number initialisation
 TRandom *r1 = new TRandom1(100);

  // Book histo for fit chisquare and chisquare probability distributions
  TH1F *p0d    = new TH1F("p0d", "fitted p0",100, -0.05, 0.05);
  //***** For case with rejecting tracks with bad chi2-probability
  //      book here another histogram p0d_rej
  TH1F *p0d_rej   = new TH1F("p0d_rej", "fitted p0 rej",100, -0.05, 0.05);

  TH1F *chi2d  = new TH1F("chi2d", "chi2-distr.",100, 0., 5.*Ndet);
  TH1F *pchi2d = new TH1F("pchi2d", "prob(chi2) distr.",100, 0.,1.);
  //***** For final try with individual hit outlier rejection:
  //      book here three new histograms p0d_iter, chi2d_iter, pchi2d_iter
  //      by copy paste
  TH1F *p0d_iter    = new TH1F("p0d_iter", "fitted p0 iter",100, -0.05, 0.05);
  TH1F *chi2di_iter  = new TH1F("chi2d_iter", "chi2-distr. iter",100, 0., 5.*Ndet);
  TH1F *pchi2di_iter = new TH1F("pchi2d_iter", "prob(chi2) distr. iter",100, 0.,1.);



   gStyle->SetOptFit(1);
   // gStyle->SetOptStat(0);

  // Loop over Tracks
  for ( Int_t itr=0; itr<Ntra; itr++){

  TH1F *a = new TH1F("a", "track",Ndet, xmin, xmax); 
  // Loop over detector hits
  for ( Int_t i=1; i<Ndet+1; i++){
 
  Double_t x;
  x = r1->Rndm(1);
  // Generate hits
  if (x>frac_noise) {
  x = r1->Gaus(0.,1.); // Signal hit
  }
  else {
  x = r1->Rndm(1); // Noise hit
  x = 10.*(-1.+2.*x); // Distribute noise hit over 10 sigma
  }
  a->SetBinContent(i,x*yerr);
  a->SetBinError(i,yerr);   
  }

  // Track fit with constant function
  TF1 *func = new TF1("func", "[0]", xmin, xmax );
  a->Fit("func");
  //cout << "chi2 = " << func->GetChisquare() << endl;
  Double_t p0   = func->GetParameter(0);
  Double_t chi2 = func->GetChisquare();
  Double_t pchi2 = func->GetProb();

   a->Draw();
  
  //***** Optional: Now you could reject bad track-fits by filling the histograms
  //                only when the pchi2 value is not too small
  //        
  if (pchi2>0.001) p0d_rej->Fill(p0);
  p0d->Fill(p0);
  chi2d->Fill(chi2);
  pchi2d->Fill(pchi2);

  //***** Final try for this track with hit outlier rejection, 
  //      Replace following pseudo code by real one:
  //      Loop over detector hits
  //        Get hit position x (Use 'GetBinContent' function)
  //        chi2i = (x-p0)**2/yerr2 // Calculate individual chi2 contribution
  //        if chi2i>5 then SetBinError(i,yerr*100) // Downweighting of outliers
  //      Endloop
  //      Repeat the trackfit
  //      Fill three histograms as above
 

// Second iteration
// Loop over detector hits
  for ( Int_t i=1; i<Ndet+1; i++){
    // Get Hits back 
    x = a->GetBinContent(i);
    // cout << "2. iter  i = " << i << "  x = " << x << endl;
    // Chisquare contribution of that hit  
    Double_t chi2_i = (p0-x)*(p0-x)/yerr2;
    // Downweight hits with large chisquare
    if (chi2_i>5) {
      //cout << "  Noise hit " << i << " x = " << x << " chi2_i  " << chi2_i << endl;
       a->SetBinContent(i,0.);
       a->SetBinError(i,0.);
    }
  }
  a->Fit("func");
  cout << "chi2 = " << func->GetChisquare() << endl;
  p0   = func->GetParameter(0);
  chi2 = func->GetChisquare();
  pchi2 = func->GetProb();    
  p0d_iter->Fill(p0);
  chi2d_iter->Fill(chi2);
  pchi2d_iter->Fill(pchi2);
 } 

  //***** Plot results:
  // p0d->Draw();
  // chi2d->Draw();
  // pchi2d->Draw();
} 
